package org.lightj.concurrent;

import java.util.concurrent.atomic.AtomicLong;

/**
 * base {@link IQueueTask}
 * 
 * @author biyu
 *
 */
public abstract class AbstractQueueTask implements IQueueTask {
	
	/** default name if not specified otherwise */
	private static final AtomicLong taskId = new AtomicLong();
	
	/** status */
	private QueueTaskStatus status;
	/** name */
	private final String name;
	/** priority */
	private final QueueTaskPriority priority;
	/** QueueTaskGroup this task associated with */
	private final QueueTaskGroup group;
	/** stats */
	private final QueueTaskStats stats = new QueueTaskStats();
	
	/** constructor */
	public AbstractQueueTask(QueueTaskGroup group, String name, QueueTaskPriority priority) {
		this.group = group;
		this.name = name;
		this.priority = priority;
	}
	
	/** constructor, used default name */
	public AbstractQueueTask(QueueTaskGroup group, QueueTaskPriority priority) {
		this(group, Long.toString(taskId.incrementAndGet()), priority);
	}
	
	/** constructor, use default name and priority */
	public AbstractQueueTask(QueueTaskGroup group) {
		this(group, Long.toString(taskId.incrementAndGet()), QueueTaskPriority.medium);
	}

	/** get name */
	public String getName() {
		return this.name;
	}

	/** get priority */
	public QueueTaskPriority getPriority() {
		return this.priority;
	}

	/** get status */
	public QueueTaskStatus getStatus() {
		return status;
	}
	
	/** set status */
	void setStatus(QueueTaskStatus status) {
		this.status = status;
	}

	/** get QueueTaskGroup */
	public final QueueTaskGroup getGroup() {
		return group;
	}

	/** get stats */
	public QueueTaskStats getStats() {
		return stats;
	}
	
	/** compare to another QueueTask */
	public int compareTo(IQueueTask o) {
		if (o == null) return -1;
		return getPriority().getPriority() - o.getPriority().getPriority();	
	}
	
	/** string description */
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append(group.getGroupName()).append('-').append(name).append('-').append(priority.name());
		return buf.toString();
	}

}
