package org.lightj.concurrent;

import java.util.ArrayList;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * circular array used for round robin algorithm
 * @author biyu
 *
 * @param <E>
 */
public class CircularArray<E> {
	
	private ArrayList<E> array;
	private AtomicInteger idx;
	private volatile int size;
	
	/** constructor */
	public CircularArray() {
		size = 0;
		idx = new AtomicInteger(0);
		array = new ArrayList<E>(size);
	}
	
	/** constructor */
	public CircularArray(E[] primedContent) {
		this();
		for (E c : primedContent) {
			add(c);
		}
	}
	
	/** add new element */
	public synchronized void add(E o) {
		array.add(o);
		array.trimToSize();
		size++;
	}
	
	/** replace the current element with a new one */
	public synchronized void replace(E o) {
		idx.compareAndSet(size, 0);
		array.set(idx.getAndIncrement(), o);
	}
	
	/** circular get next element from array */
	public E next() {
		idx.compareAndSet(size, 0);
		return array.get(idx.getAndIncrement());
	}
	
	/** size */
	public int size() {
		return size;
	}

}
