package org.lightj.concurrent;

import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

/**
 * compound iterator
 * 
 * @author biyu
 *
 */
class CombinedIterator<E> implements Iterator<E> {
	
	private List<Iterator<E>> iterators;
	private Iterator<E> current;
	public CombinedIterator(List<Iterator<E>> iterators) {
		if (iterators == null || iterators.isEmpty()) {
			throw new IllegalArgumentException("source iterator cannot be empty");
		}
		this.iterators = iterators;
	}

	public boolean hasNext() {
		current = null;
		for (Iterator<E> iter : iterators) {
			if (iter.hasNext()) {
				current = iter;
				return true;
			}
		}
		return false;
	}

	public E next() {
		if (current == null) {
			for (Iterator<E> iter : iterators) {
				if (iter.hasNext()) {
					current = iter;
				}
			}
		}
		if (current == null) throw new NoSuchElementException();
		return current.next();
 	}

	public void remove() {
		if (current == null) throw new NoSuchElementException();
		current.remove();
	}
	
}