package org.lightj.concurrent;

import java.util.Collection;
import java.util.Iterator;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.TimeUnit;

/**
 * composite blocking queue that wrap a blocking queue and aggregate more functionalities like Stats
 * 
 * @author biyu
 *
 * @param <E>
 */
public class CompositeBlockingQueue<E> implements BlockingQueue<E> {
	
	/** internal queue */
	private BlockingQueue<E> queue;
	
	/** queue stats */
	private QueueStats stats;
	
	/** constructor */
	public CompositeBlockingQueue(String name, BlockingQueue<E> queue) {
		this.queue = queue;
		stats = new QueueStats(name, queue);
	}

	public boolean add(E o) {
		return queue.add(o);
	}

	public int drainTo(Collection<? super E> c) {
		return queue.drainTo(c);
	}

	public int drainTo(Collection<? super E> c, int maxElements) {
		return queue.drainTo(c);
	}

	public boolean offer(E o) {
		return queue.offer(o);
	}

	public boolean offer(E o, long timeout, TimeUnit unit) throws InterruptedException {
		return queue.offer(o, timeout, unit);
	}

	public E poll(long timeout, TimeUnit unit) throws InterruptedException {
		return queue.poll(timeout, unit);
	}

	public void put(E o) throws InterruptedException {
		queue.put(o);
	}

	public int remainingCapacity() {
		return queue.remainingCapacity();
	}

	public E take() throws InterruptedException {
		return queue.take();
	}

	public E element() {
		return queue.element();
	}

	public E peek() {
		return queue.peek();
	}

	public E poll() {
		return queue.poll();
	}

	public E remove() {
		return queue.remove();
	}

	public boolean addAll(Collection<? extends E> c) {
		return queue.addAll(c);
	}

	public void clear() {
		queue.clear();
	}

	public boolean contains(Object o) {
		return queue.contains(o);
	}

	public boolean containsAll(Collection<?> c) {
		return queue.containsAll(c);
	}

	public boolean isEmpty() {
		return queue.isEmpty();
	}

	public Iterator<E> iterator() {
		return queue.iterator();
	}

	public boolean remove(Object o) {
		return queue.remove(o);
	}

	public boolean removeAll(Collection<?> c) {
		return queue.removeAll(c);
	}

	public boolean retainAll(Collection<?> c) {
		return queue.retainAll(c);
	}

	public int size() {
		return queue.size();
	}

	public Object[] toArray() {
		return queue.toArray();
	}

	public <T> T[] toArray(T[] a) {
		return queue.toArray(a);
	}
	
	public QueueStats getStats() {
		return stats;
	}

}
