package org.lightj.concurrent;

import java.util.Date;
import java.util.concurrent.atomic.AtomicLong;

import org.lightj.util.DateUtil;
import org.lightj.util.nanoxml.XMLElement;


/**
 * Stat class used for counting something
 * 
 * @author biyu
 *
 */
public abstract class CounterStats {
	
	/** type of stats */
	protected String type;

	/** name */
	private final String name;
	
	/** total execution count */
	private final AtomicLong totalCount = new AtomicLong();
	
	/** last time data gets reset */
	private Date lastResetTime;
	
	/** start time */
	private Date startTime;
	
	public CounterStats(String name, String type) {
		Date now = new Date();
		this.name = name;
		this.type = type;
		lastResetTime = now;
		startTime = now;
	}

	/** increment total execution counter */
	public final long incTotalCount() {
		return totalCount.incrementAndGet();
	}
	
	/** total execution counter */
	public final long getTotalCount() {
		return totalCount.get();
	}

	/** get the name */
	public String getName() {
		return name;
	}

	/** reset all stats */
	public synchronized void reset() {
		totalCount.set(0);
		lastResetTime = new Date();
	}

	/** export stats as xml */
	public XMLElement asXml() {
		XMLElement statElem = new XMLElement();
		statElem.setName(type);
		asXml(statElem);
		return statElem;
	}
	
	/** export stats as xml */
	protected void asXml(XMLElement statElem) {
		statElem.setAttribute("name", getName());
		statElem.setAttribute("dateStarted", DateUtil.format(startTime, "yyyy.MM.dd HH:mm:ss:SSS zzz"));
		statElem.setAttribute("resetTime", DateUtil.format(lastResetTime, "yyyy.MM.dd HH:mm:ss:SSS zzz"));
		statElem.setAttribute("executionCount", totalCount.get());
	}
}
