package org.lightj.concurrent;

import java.util.Iterator;
import java.util.NoSuchElementException;


/**
 * An iterator based on an array with circular read behavior, et. which read position reaches the end of the
 * array, it starts from head of the array again. The array is immutable, et. it can not be extended or shrinked
 * once it is created.
 * 
 * @author biyu
 *
 * @param <E>
 */
public class ImmutableCircularArrayIterator<E> implements Iterator<E> {

	/**
	 * current read position
	 */
	private int pos;
	
	/**
	 * size of the array
	 */
	private int size;
	
	/**
	 * array of data
	 */
	private E[] data = null;
	
	/**
	 * constructor, construct a circular iterator based on an existing array
	 * @param array
	 */
	public ImmutableCircularArrayIterator(E[] array) {
		super();
		data = array;
		pos = 0;
		size = data.length;
	}

	/**
	 * circular array iterator always has next unless it has a zero size array
	 */
	public boolean hasNext() {
		return size > 0;
	}

	/**
	 * return the next item in the array
	 */
	public synchronized E next() {
		if (size <= 0) throw new NoSuchElementException();
		if (pos>=size) pos -= size;
		return data[pos++];
	}

	/**
	 * DO NOT USE! IT WILL THROW RUNTIME EXCEPTION!
	 * this is an immutable iterator, it doesn't support add/remove
	 */
	public void remove() {
		throw new RuntimeException("Cannot remove item from ImmutableCircularArrayIterator");
	}

}
