package org.lightj.concurrent;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Category;
import org.apache.log4j.Logger;
import org.lightj.util.nanoxml.XMLElement;


/**
 * stats associated with {@link PriorityRoundRobinThreadPoolExecutor}
 * 
 * @author biyu
 *
 */
public class PRRThreadPoolExecutorStats {
	
	static Category logger = Logger.getLogger(PRRThreadPoolExecutorStats.class);
	
	/** singleton */
	private static final PRRThreadPoolExecutorStats s_instance = new PRRThreadPoolExecutorStats();
	
	/** singleton */
	public static PRRThreadPoolExecutorStats getInstance() {
		return s_instance;
	}
	
	/** constructor */
	private PRRThreadPoolExecutorStats() {
	}
	
	public XMLElement asXml(){
		XMLElement statElem = new XMLElement();
		statElem.setName("PRRThreadPoolExecutorStats");
		// add pool level stats
		statElem.addChild(getPoolStats()); // now
		// underneath queue stats
		statElem.addChild(getQueueStats());
		// group level stats
		XMLElement groupsElem = new XMLElement();
		groupsElem.setName("GroupStats");
		statElem.addChild(groupsElem);
		for (XMLElement groupElem : getGroupStats()) {
			groupsElem.addChild(groupElem);
		}
		return statElem;
	}
	
	/** refresh history, keep the latest x number of hours */
	void resetAfterHeartbeat() {
		ThreadPoolStats tstats = PriorityRoundRobinThreadPoolExecutor.getInstance().getStats();
		QueueStats qstats = PriorityRoundRobinThreadPoolExecutor.getInstance().getQueueStats();
		try {
			final String data =
				"prrPoolSize="+tstats.getPoolSize()+
				"&prrActiveCount="+tstats.getActiveCount()+
				"&prrCompletedCount="+tstats.getTotalCount()+
				"&prrqAvgWaitTime="+qstats.getAvgWaitTime()+
				"&prrqWorstWaitTime="+qstats.getWorstWaitTime()+
				"&prrqTotalCount="+qstats.getTotalCount();			
			logger.debug(data);
		}
		finally {
			tstats.reset();
			qstats.reset();
			for (QueueTaskGroup group : QueueTaskGroup.getAllGroups()) 
			{
				group.getStats().reset();
			}
		}
	}
	
	/** get stats from all groups */
	private List<XMLElement> getGroupStats() {
		ArrayList<XMLElement> history = new ArrayList<XMLElement>();
		for (QueueTaskGroup group : QueueTaskGroup.getAllGroups()) 
		{
			history.add(group.getStats().asXml());
		}
		return history;
	}
	
	/** get stats for pool */
	private XMLElement getPoolStats() {
		XMLElement elem = PriorityRoundRobinThreadPoolExecutor.getInstance().getStats().asXml();
		return elem;
	}
	
	/** get stats for queue */
	private XMLElement getQueueStats() {
		XMLElement elem = PriorityRoundRobinThreadPoolExecutor.getInstance().getQueueStats().asXml();
		return elem;
	}
	
}
