package org.lightj.concurrent;

import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.lightj.config.LightJRuntimeConfigs;

/**
 * A threadpool executor that poll its tasks from {@link PriorityRoundRobinBlockingQueue}
 * 
 * @author biyu
 *
 */
public class PriorityRoundRobinThreadPoolExecutor extends
		AbstractThreadPoolExecutor {
	
	/** singleton */
	private static PriorityRoundRobinThreadPoolExecutor singleton;
	
	/** singleton */
	public static synchronized final PriorityRoundRobinThreadPoolExecutor getInstance() {
		if (singleton == null) {
			PriorityRoundRobinBlockingQueue queue = new PriorityRoundRobinBlockingQueue();
			CompositeBlockingQueue<Runnable> compositeQueue = new CompositeBlockingQueue<Runnable>("PriorityRoundRobinQueue", queue);
			// we don't need to see queue content
			compositeQueue.getStats().setDumpSize(0);
			queue.getRoundRobinQueue().setAlgorithm(new RoundRobinQueueAlgorithmImpl());
			singleton = new PriorityRoundRobinThreadPoolExecutor(compositeQueue);
			singleton.setStats(new ThreadPoolStats("PriorityRoundRobinThreadPool", singleton));
			ThreadPoolComponentStatus.getInstance();
		}
		return singleton;
	}
	
	/**
	 * constructor
	 *
	 */
	private PriorityRoundRobinThreadPoolExecutor(CompositeBlockingQueue<Runnable> queue) {
		super(LightJRuntimeConfigs.coreThreadSize, 
				LightJRuntimeConfigs.maxThreadSize, 
				LightJRuntimeConfigs.maxIdleTimeSec, 
				TimeUnit.SECONDS, 
				queue, 
				Executors.defaultThreadFactory());
		prestartAllCoreThreads();
	}

}
