package org.lightj.concurrent;

import java.util.Queue;
import java.util.concurrent.TimeUnit;

import org.lightj.util.nanoxml.XMLElement;


/**
 * collect stats for a queue
 * 
 * @author biyu
 *
 */
@SuppressWarnings("rawtypes")
public class QueueStats extends CounterStats {
	
	/** queue it collect stats from */
	private Queue queue;
	
	/** how many we dump into the report */
	private int dumpSize = 100;
	
	/** average wait time in ms */
	private long avgWaitTime = 0L;
	
	/** worst wait time in ms */
	private long worstWaitTime = 0L;

	public QueueStats(String name, Queue queue) {
		super(name, "QueueStats");
		this.queue = queue;
	}

	/** export stats as xml */
	protected void asXml(XMLElement statElem) {
		super.asXml(statElem);
		statElem.setAttribute("size", queue.size());
		statElem.setAttribute("averageWaitTime", TimeUnit.MILLISECONDS.convert(avgWaitTime, TimeUnit.NANOSECONDS));
		statElem.setAttribute("worstWaitTime", TimeUnit.MILLISECONDS.convert(worstWaitTime, TimeUnit.NANOSECONDS));
		Object[] contents = queue.toArray();
		int count = 0, len = contents.length;
		XMLElement contentsElem = new XMLElement();
		contentsElem.setName("QueueContents");
		statElem.addChild(contentsElem);
		while (count < dumpSize && count < len) {
			if ( contents[count]!=null ) {
				XMLElement elem = new XMLElement();
				elem.setName("QueueContent");
				elem.setContent(contents[count].toString());
				contentsElem.addChild(elem);
			}
			count++;
		}
	}

	/** reset all stats */
	public synchronized void reset() {
		avgWaitTime = 0L;
		worstWaitTime = 0L;
		super.reset();
	}
	
	/**
	 * recalcuate waittime (worst and mean)
	 * @param executionWaitTime
	 */
	public synchronized void adjustWaitTime(long executionWaitTime) {
		avgWaitTime += ((executionWaitTime-avgWaitTime)/(getTotalCount()+1));
		worstWaitTime = Math.max(worstWaitTime, executionWaitTime);
	}
	
	/** set dump size */
	public void setDumpSize(int dumpSize) {
		this.dumpSize = dumpSize;
	}

	public final long getAvgWaitTime() {
		return avgWaitTime;
	}

	public final long getWorstWaitTime() {
		return worstWaitTime;
	}

}
