package org.lightj.concurrent;

import java.util.AbstractQueue;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Queue;

/**
 * An unbounded round robin queue consists of priority queue(s), tasks are polled in the order of
 * firstly round robin among queues determined by actively executing work in the reverse order,
 * secondly priority based with a specific priority queue.
 * 
 * Not thread safe, thread safety has to be guarded externally 
 * 
 * A priorityQueue doesn't allow null element
 * 
 * @author biyu
 *
 */
public class RoundRobinPriorityQueue extends AbstractQueue<Runnable> {
	
	/** offer and poll algorithm */ 
	private RoundRobinQueueAlgorithm<Runnable> algorithm;

	/**
	 * constructor
	 */
	RoundRobinPriorityQueue() {
	}

	@Override
	public Iterator<Runnable> iterator() {
		ArrayList<Iterator<Runnable>> iterators = new ArrayList<Iterator<Runnable>>();
		for (Queue<Runnable> queue : algorithm.getAllQueues()) {
			iterators.add(queue.iterator());
		}
		return new CombinedIterator<Runnable>(iterators);
	}

	@Override
	public int size() {
		int size = 0;
		for (Queue<Runnable> q : algorithm.getAllQueues()) {
			size += q.size();
		}
		return size;
	}

	/**
	 * enqueue an object
	 */
	public boolean offer(Runnable o) {
		return algorithm.offer(o);
	}

	/**
	 * peek queue
	 */
	public Runnable peek() {
		return algorithm.peek();
	}

	/**
	 * poll the next element from queue
	 */
	public Runnable poll() {
		return algorithm.poll();
	}
	
	/** set algorith */
	final void setAlgorithm(RoundRobinQueueAlgorithm<Runnable> algorithm) {
		this.algorithm = algorithm;
	}

}
