package org.lightj.concurrent;

import java.util.Collection;
import java.util.Queue;

public class RoundRobinQueueAlgorithmImpl implements RoundRobinQueueAlgorithm<Runnable> {

	public Runnable poll() {
		Runnable task = null;
		for (int i = 0, size = QueueTaskGroup.rrArrays.size(); i < size; i++) {
			task = QueueTaskGroup.rrArrays.next().poll();
			if (task != null) return task;
		}
		return task;
	}

	/** pick a queue to offer */
	public boolean offer(Runnable o) {
		if (!(o instanceof IQueueTask)) throw new IllegalArgumentException("Only QueueTask is supported");
		IQueueTask task = (IQueueTask) o;
		if (QueueTaskGroup.rrQueues.containsKey(task.getGroup())) {
			return QueueTaskGroup.rrQueues.get(task.getGroup()).offer(o);
		}
		else {
			throw new RuntimeException("Task group " + task.getGroup().getGroupName() + " is not allowed in the queue");
		}
	}

	/** pick a queue to peek */
	public Runnable peek() {
		Runnable task = null;
		for (int i = 0, size = QueueTaskGroup.rrArrays.size(); i < size; i++) {
			task = QueueTaskGroup.rrArrays.next().poll();
			if (task != null) return task;
		}
		return task;
	}

	/** all internal queues */
	public Collection<? extends Queue<Runnable>> getAllQueues() {
		return QueueTaskGroup.rrQueues.values();
	}
	
}
