package org.lightj.concurrent;

import java.util.Collections;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.log4j.Category;
import org.apache.log4j.Logger;
import org.lightj.util.nanoxml.XMLElement;


/**
 * component status for thread pools
 * 
 * @author biyu
 *
 */
public class ThreadPoolComponentStatus extends TimerTask {
	
	static Category logger = Logger.getLogger(ThreadPoolComponentStatus.class);
	private static final String TASK_LABEL = "ThreadPoolComponentStatusRefreshTask";
	private static final long REFRESH_INTERVAL = 60 * 60 * 1000L; // 60 minutes

	/** singleton */
	private static ThreadPoolComponentStatus s_instance = null;

	/** constructor */
	private ThreadPoolComponentStatus(){
		// create a timer to refresh the cache based on the config bean
		Timer t = new Timer(TASK_LABEL, true);
		// delay the first cache refresh 
		t.schedule(this, REFRESH_INTERVAL, REFRESH_INTERVAL);
	}
	
	/** singleton */
	public static final ThreadPoolComponentStatus getInstance() {
		if (s_instance == null) {
			s_instance = new ThreadPoolComponentStatus();
		}
		return s_instance;
	}
	
	/** property to xml */
	public synchronized XMLElement getPropertiesAsXml() {
		XMLElement root = new XMLElement();
		root.setName(getName());
		root.setAttribute("categoryId", "lightj.threadpool");
		root.addChild(PRRThreadPoolExecutorStats.getInstance().asXml());
		if(root.getChildren().isEmpty()){
			root.setContent("No Thread Pool is Enabled");
		}
		return root;
	}

	/**
	 * collect stats
	 *
	 */
	private void resetAfterHeartbeat() {
		PRRThreadPoolExecutorStats.getInstance().resetAfterHeartbeat();
	}

	/** alias */
	public String getAlias() {
		return getName();
	}

	/** name */
	public String getName() {
		return "eko.ThreadPool";
	}

	/** properties */
	@SuppressWarnings("rawtypes")
	public List getProperties() {
		return Collections.EMPTY_LIST;
	}

	/** status */
	public String getStatus() {
		return "CREATED";
	}

	@Override
	public void run() {
		resetAfterHeartbeat();
	}

}
