package org.lightj.config;

import java.net.URL;

import org.lightj.RuntimeContext;
import org.lightj.initialization.BaseInitializable;
import org.lightj.initialization.BaseModule;
import org.lightj.initialization.InitializationProcessor;

import com.thoughtworks.xstream.InitializationException;

/**
 * initialize application configs
 * see {@link InitializationProcessor}
 * 
 * @author biyu
 *
 */
public class ConfigModule {

	/** inner module for the real init */
	private static ConfigModuleInner s_Module = null;
	
	/**
	 * constructor
	 *
	 */
	public ConfigModule() {
		init();
	}
	
	/**
	 * init singleton just once
	 *
	 */
	private synchronized void init() {
		if (s_Module == null) {
			s_Module = new ConfigModuleInner();
		}
	}
	
	/**
	 * initializable module
	 * @return
	 */
	public BaseModule getModule() {
		return s_Module;
	}
	
	/**
	 * load configs
	 * @param configRoot
	 * @param product
	 * @param configs
	 * @return
	 */
	public ConfigModule loadConfigs(final URL configRoot, final String product, final IRuntimeConfigs configs) 
	{
		s_Module.addInitializable(new BaseInitializable() {
			@Override
			protected void initialize() {
				ConfigManager.init(configRoot, product, configs);
			}

			@Override
			protected void shutdown() {
			}

		});
		return this;
	}
	
	/**
	 * load configs
	 * @param product
	 * @param configs
	 * @return
	 */
	public ConfigModule loadConfigs(final String product, final IRuntimeConfigs configs) 
	{
		s_Module.addInitializable(new BaseInitializable() {
			@Override
			protected void initialize() {
				if (RuntimeContext.getConfigRoot() == null) {
					throw new InitializationException("Configuration root not set");
				}
				
				ConfigManager.init(RuntimeContext.getConfigRoot(), product, configs);
			}

			@Override
			protected void shutdown() {
			}

		});
		return this;
	}

	/**
	 * load config from xml string
	 * @param xmlString
	 * @param configs
	 * @return
	 */
	public ConfigModule loadConfigsFromString(final String xmlString, final IRuntimeConfigs configs) {
		s_Module.addInitializable(new BaseInitializable() {
			@Override
			protected void initialize() {

				ConfigManager.init(xmlString, configs);
			}

			@Override
			protected void shutdown() {
			}

		});
		return this;
	}

	/**
	 * inner module
	 * @author biyu
	 *
	 */
	private class ConfigModuleInner extends BaseModule {

		private ConfigModuleInner() {
			super(new BaseModule[] {});
		}
		
	}
}
