package org.lightj.config;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.lightj.util.nanoxml.XMLElement;

/**
 * config manager interface, file based configuraiton
 * @author biyu
 *
 */
@SuppressWarnings("rawtypes")
public interface IConfigManager {
	
	/**
	 * what environment application belongs to
	 * @return
	 */
	public EnvironmentEnum deriveEnvironment();

	/**
	 * override a property at runtime
	 * @param path
	 * @param value
	 */
	public void overrideProperty(String path, String value);
	

	/**
	 * if a property exists
	 * 
	 * @param path
	 * @return
	 */
	public boolean exists(String path);

	/**
	 * get a boolean property
	 * 
	 * @param path
	 * @return
	 */
	public boolean getBoolean(String path);

	/**
	 * get an int property
	 * 
	 * @param path
	 * @return
	 */
	public int getInt(String path);

	/**
	 * get a float property
	 * 
	 * @param path
	 * @return
	 */
	public double getFloat(String path);

	/**
	 * get a string property
	 * 
	 * @param path
	 * @return
	 */
	public String getString(String path);

	/**
	 * get a list of string properties
	 * 
	 * @param path
	 * @return
	 */
	public List getList(String path);

	/**
	 * get name/value pairs as a map
	 * 
	 * @param path
	 * @return
	 */
	public Map getMap(String path);

	/**
	 * get class
	 * 
	 * @param path
	 * @return
	 */
	public Class getClass(String path);

	/**
	 * if a property exists
	 * 
	 * @param propSet
	 * @param propName
	 * @return
	 */
	public boolean exists(String propSet, String propName);

	/**
	 * get a boolean property
	 * 
	 * @param propSet
	 * @param propName
	 * @param defaultValue
	 * @return
	 */
	public boolean getBoolean(String propSet, String propName, boolean defaultValue);

	/**
	 * get an int property
	 * 
	 * @param propSet
	 * @param propName
	 * @param defaultValue
	 * @return
	 */
	public int getInt(String propSet, String propName, int defaultValue);
	
	/**
	 * get a float property
	 * 
	 * @param propSet
	 * @param propName
	 * @param defaultValue
	 * @return
	 */
	public double getFloat(String propSet, String propName, double defaultValue);

	/**
	 * get a string property
	 * 
	 * @param propSet
	 * @param propName
	 * @param defaultValue
	 * @return
	 */
	public String getString(String propSet, String propName, String defaultValue);

	/**
	 * get a list of string properties
	 * 
	 * @param propSet
	 * @param propName
	 * @param defaultList
	 * @return
	 */
	public ArrayList getList(String propSet, String propName, ArrayList defaultList);

	/**
	 * get name/value pair as a map
	 * 
	 * @param propSet
	 * @param propName
	 * @param defaultMap
	 * @return
	 */
	public HashMap getMap(String propSet, String propName, HashMap defaultMap);

	/**
	 * get class
	 * 
	 * @param propSet
	 * @param propName
	 * @param defaultClass
	 * @return
	 */
	public Class getClass(String propSet, String propName, Class defaultClass);
	
	/**
	 * get object
	 * 
	 * @param propSet
	 * @param propName
	 * @param defaultClass
	 * @return
	 */
	public Object getObject(String propSet, String propName, Object defaultClass);

	/**
	 * get all configs as a flat map
	 * 
	 * @return
	 */
	public HashMap getFlatMap();

	/**
	 * convert all configs to a flat map
	 * 
	 * @param pm
	 * @param flatMap
	 * @param prefix
	 */
	public void convertToFlatMap(HashMap pm, HashMap flatMap, String prefix);

	/**
	 * render an xml to be used in component status page
	 * 
	 * @return
	 */
	public XMLElement asXml();



	/**
	 * render list of map in component status page
	 * @return
	 */
	public List getComponentStatusProperties();


}
