package org.lightj.example.task;

import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;

import org.apache.log4j.Category;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.lightj.task.ITaskListener;
import org.lightj.task.Task;
import org.lightj.task.TaskResult;
import org.lightj.util.ConcurrentUtil;

public class SimpleTaskListener implements ITaskListener {
	
	/** logger */
	static Category logger = Logger.getLogger(SimpleTaskListener.class);
	
	/** pause lock */
	protected ReentrantLock lock;
	
	/** pause condition */
	protected Condition cond;

	/** result */
	protected TaskResult result;
	
	/** when to unlock */
	protected boolean unlockOnComplete = true;
	
	/** constructor */
	public SimpleTaskListener() {}
	
	/** constructor with a lock */
	public SimpleTaskListener(ReentrantLock lock, Condition cond, boolean unlockOnComplete) {
		this.lock = lock;
		this.cond = cond;
		this.unlockOnComplete = unlockOnComplete;
	}

	/** task result */
	public void handleTaskResult(TaskResult result) {
		logger.log(Level.INFO, "DummyListener Task Status = " + result.getStatus() + ", Properties = " + result.getMeta());
		logger.log(Level.INFO, "DummyListener Logs = " + result.getMsg() );
		this.result = result;
		if (!unlockOnComplete && lock != null) {
			ConcurrentUtil.signalAll(lock, cond);
		}
	}
	
	public TaskResult getResult() {
		return result;
	}

	public void setResult(TaskResult result) {
		this.result = result;
	}

	public void taskSubmitted(Task task) {
		logger.log(Level.INFO, "Dummy listerner called with task submitted TaskId = " + task.getTaskId());
	}

	public void taskCreated(Task task) {
		logger.log(Level.INFO, "Dummy listerner called with task created TaskId = " + task.getTaskId());
	}

	public void taskCompleted(Task task) {
		logger.log(Level.INFO, "Dummy listerner called with task completed TaskId = " + result.getTaskId());
		if (unlockOnComplete && lock != null) {
			ConcurrentUtil.signalAll(lock, cond);
		}
	}

}
