package org.lightj.jms;

import java.util.concurrent.ExecutorService;

import javax.jms.ExceptionListener;
import javax.jms.MessageListener;
import javax.jms.Session;

/**
 * all properties for a session
 * @author biyu
 *
 */
public class JmsConfig {

	///////////////// jms connection information ///////////////
	private String providerUrl;
	private String username;
	private String password;
	private ExceptionListener exHandler;
	private boolean isQueue = false;
	private String name;
	
	///////////////// jms session information ////////////////
	///////////////// for consuming ///////////////
	private boolean consuming;
	private String msgSelector;
	private MessageListener msgListener;
	/** applicable only for topic session */
	private boolean noLocal;
	/** applicable only for topic session */
	private boolean durable;
	/** executor used as jms consumer
	 * IMPORTANT : do NOT use multithread consuming unless ordering to your event is NOT important in a small scale!! */ 
	private ExecutorService consumerExecutor;
	/** consuming acknowledge */
	private int consumingAck = Session.AUTO_ACKNOWLEDGE; 

	/////////////////// for producing //////////////
	private boolean producing;
	/** IMPORTANT : do NOT use multithread publishing unless ordering to your event is NOT important in a small scale!! */ 
	private int numOfProducingThread;
	/** time to live */
	private long producingTtl;
	
	public ExceptionListener getExHandler() {
		return exHandler;
	}
	public boolean isQueue() {
		return isQueue;
	}
	public String getPassword() {
		return password;
	}
	public String getProviderUrl() {
		return providerUrl;
	}
	public String getUsername() {
		return username;
	}
	public boolean isConsuming() {
		return consuming;
	}
	public boolean isDurable() {
		return durable;
	}
	public MessageListener getMsgListener() {
		return msgListener;
	}
	public String getMsgSelector() {
		return msgSelector;
	}
	public String getName() {
		return name;
	}
	public boolean isNoLocal() {
		return noLocal;
	}
	public int getNumOfProducingThread() {
		return numOfProducingThread;
	}
	public boolean isProducing() {
		return producing;
	}
	public ExecutorService getConsumerExecutor() {
		return consumerExecutor;
	}
	public long getProducingTtl() {
		return producingTtl;
	}
	public int getConsumingAck() {
		return consumingAck;
	}
	
	public JmsConfig connectToQueue(
			String providerUrl,
			String qName,
			String username,
			String password,
			ExceptionListener exHandler) {
		this.providerUrl = providerUrl;
		this.name = qName;
		this.username = username;
		this.password = password;
		this.exHandler = exHandler;
		this.isQueue = true;
		return this;
	}
	
	public JmsConfig connectToTopic(
			String providerUrl,
			String tName,
			String username,
			String password,
			ExceptionListener exHandler) {
		this.providerUrl = providerUrl;
		this.name = tName;
		this.username = username;
		this.password = password;
		this.exHandler = exHandler;
		this.isQueue = false;
		return this;
	}
	
	public JmsConfig produce(
			int numOfProducingThread,
			long producingTtl) 
	{
		this.producing = true;
		this.numOfProducingThread = numOfProducingThread;
		this.producingTtl = producingTtl;
		return this;
	}
	
	public JmsConfig consume(
			MessageListener msgListener,
			String msgSelector,
			boolean noLocal,
			boolean durable,
			int consumingAck,
			ExecutorService consumingExecutor) 
	{
		this.consuming = true;
		this.msgListener = msgListener;
		this.msgSelector = msgSelector;
		this.noLocal = noLocal;
		this.durable = durable;
		this.consumingAck = consumingAck;
		this.consumerExecutor = consumingExecutor;
		return this;
	}
	
}
