package org.lightj.jms;

import java.util.HashSet;
import java.util.Set;

import javax.jms.JMSException;
import javax.naming.NamingException;

import org.lightj.initialization.BaseInitializable;
import org.lightj.initialization.BaseModule;
import org.lightj.initialization.InitializationException;
import org.lightj.util.Log4jProxy;

public class JmsModule {
	
	static Log4jProxy logger = Log4jProxy.getInstance(JmsModule.class);
	
	private static JmsModuleInner module = null;
	
	public JmsModule() {
		init();
	}
	
	private synchronized void init() {
		if (module == null) module = new JmsModuleInner();
	}
	
	public BaseModule getModule() {
		return module;
	}
	
	public JmsModule addJmsConfig(JmsConfig cfg) {
		module.jmsCfgs.add(cfg);
		return this;
	}
	
	private class JmsModuleInner extends BaseModule {
		
		private Set<JmsConfig> jmsCfgs = new HashSet<JmsConfig>();
		private JmsModuleInner() {
			super(new BaseModule[] {});
			addInitializable(new BaseInitializable() {
				
				@Override
				protected void initialize() {

					try {
						for (JmsConfig jmsCfg : jmsCfgs) {
							MessageManager.getInstance().setup(jmsCfg);
						}
					} catch (JMSException e) {
						logger.fatal("Messaging failed to initialize", e);
						throw new InitializationException(e);
					} catch (NamingException e) {
						logger.fatal("Messaging failed to initialize", e);
						throw new InitializationException(e);
					}
				}

				@Override
				protected void shutdown() {
					MessageManager.getInstance().shutdown();
				}
				
			});
			
		}
		
	}

}
