package org.lightj.jms;

import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.jms.Message;
import javax.jms.Session;

import org.lightj.util.StringUtil;


/**
 * outbound message
 * @author biyu
 *
 */
public class OutboundMessageImpl implements OutboundMessage {

	static SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd kk:mm:ss.SSS");

	private String m_JMSMessageID;
	private int m_JMSDeliveryMode;
	private long m_JMSExpiration;
	private int m_JMSPriority;
	private long m_JMSTimestamp;
	private String m_JMSCorrelationID;
	private String m_JMSContent;
	private Map<String, Object> m_JMSProperties;
	
	private OutboundMessageImpl() {		
		 m_JMSProperties = new HashMap<String, Object>();
	}
	
	public String getM_JMSCorrelationID() {
		return m_JMSCorrelationID;
	}
	public void setM_JMSCorrelationID(String correlationID) {
		m_JMSCorrelationID = correlationID;
	}
	public int getM_JMSDeliveryMode() {
		return m_JMSDeliveryMode;
	}
	public void setM_JMSDeliveryMode(int deliveryMode) {
		m_JMSDeliveryMode = deliveryMode;
	}
	public long getM_JMSExpiration() {
		return m_JMSExpiration;
	}
	public void setM_JMSExpiration(long expiration) {
		m_JMSExpiration = expiration;
	}
	
	public String getM_JMSMessageID() {
		return m_JMSMessageID;
	}
	public void setM_JMSMessageID(String messageID) {
		m_JMSMessageID = messageID;
	}
	public int getM_JMSPriority() {
		return m_JMSPriority;
	}
	public void setM_JMSPriority(int priority) {
		m_JMSPriority = priority;
	}
	public long getM_JMSTimestamp() {
		return m_JMSTimestamp;
	}
	public void setM_JMSTimestamp(long timestamp) {
		m_JMSTimestamp = timestamp;
	}
	public String getM_JMSContent() {
		return m_JMSContent;
	}
	public void setM_JMSContent(String content) {
		m_JMSContent = content;
	}
	public Object getM_JMSProperty(String name) {
		return m_JMSProperties.get(name);
	}
	public Map<String, Object> getReadOnlyJMSProperties() {
		return Collections.unmodifiableMap(m_JMSProperties);
	}
	public void setM_JMSProperty(String name, Object value) {
		m_JMSProperties.put(name, value);
	}

	/**
	 * create a new instance
	 * @param messageID default null
	 * @param deliveryMode -1 will use default {@link Session#AUTO_ACKNOWLEDGE}
	 * @param expiration millisec after timestamp to expire, -1 will use default {@link Message#DEFAULT_TIME_TO_LIVE}
	 * @param priority -1 will use default {@link Message#DEFAULT_PRIORITY}
	 * @param timeStamp -1 will use default {@link System#currentTimeMillis()}
	 * @param correlationId -1 will use default {@link MessageManager#genMessageId()}
	 * @param content
	 * @return
	 */
	public static OutboundMessageImpl newInstance(
			String messageID,
			int deliveryMode,
			long expiration,
			int priority,
			long timeStamp,
			String correlationId,
			String content){
		OutboundMessageImpl msg = new OutboundMessageImpl();
		if (timeStamp <= 0) {
			timeStamp = System.currentTimeMillis();
		}
		msg.setM_JMSMessageID(messageID);
		msg.setM_JMSDeliveryMode(deliveryMode>=0 ? deliveryMode : Session.AUTO_ACKNOWLEDGE);
		msg.setM_JMSExpiration(expiration>=0 ? timeStamp + expiration : MessageManager.getInstance().getDefExpiration());
		msg.setM_JMSPriority(priority>=0 ? priority : Message.DEFAULT_PRIORITY);
		msg.setM_JMSTimestamp(timeStamp);
		msg.setM_JMSContent(content);
		msg.setM_JMSCorrelationID(correlationId!=null ? correlationId : MessageManager.genMessageId());
		return msg;
	}

	/**
	 * create a new message
	 * @param expiration
	 * @param content
	 * @return
	 */
	public static OutboundMessageImpl newInstance(long expiration, String correlationID, String content) {
		OutboundMessageImpl msg = new OutboundMessageImpl();
		msg.setM_JMSExpiration(expiration>=0 ? System.currentTimeMillis() + expiration : MessageManager.getInstance().getDefExpiration());
		msg.setM_JMSContent(content);
		msg.setM_JMSCorrelationID(correlationID);
		return msg;
	}
	
	/**
	 * display
	 */
	public String toString() {
		StringBuffer buf = new StringBuffer();
		if (m_JMSTimestamp>0) buf.append("Time=").append(fmt.format(new Date(m_JMSTimestamp))).append(',');
		buf.append("CorrelationID=").append(m_JMSCorrelationID).append(',');
		buf.append("Content=").append(StringUtil.trimToLength(m_JMSContent, 40)).append("...");
		return buf.toString();
	}

}