package org.lightj.locking;

import java.sql.SQLException;

import org.lightj.dal.BaseDatabaseType;
import org.lightj.dal.ConnectionHelper;
import org.lightj.dal.DatabaseModule;
import org.lightj.dal.HsqlDatabaseType;
import org.lightj.initialization.BaseInitializable;
import org.lightj.initialization.BaseModule;
import org.lightj.initialization.InitializationException;
import org.lightj.initialization.ShutdownException;
import org.lightj.locking.GlobalSemaphoreManager.GlobalSemaphoreImpl;
import org.lightj.locking.dal.GlobalSemaphoreDAO;

public class LockingModule {
	
	private static LockingModuleInner module = new LockingModuleInner();
	
	public LockingModule() {
	}
	
	public BaseModule getModule() {
		return module;
	}
	
	public LockingModule setGlobalSemaphore(IGlobalSemaphore semaphoreImpl) {
		module.setSemaphoreImpl(semaphoreImpl);
		return this;
	}
	
	public IGlobalSemaphore getGlobalSemaphore() {
		return module.semaphoreImpl;
	}
	
	private static class LockingModuleInner extends BaseModule {
		
		private IGlobalSemaphore semaphoreImpl;
		private void setSemaphoreImpl(IGlobalSemaphore semaphoreImpl) {
			this.semaphoreImpl = semaphoreImpl;
			if (semaphoreImpl instanceof GlobalSemaphoreImpl &&
					((GlobalSemaphoreImpl) semaphoreImpl).getDb() != null) {
				BaseDatabaseType db = ((GlobalSemaphoreImpl) semaphoreImpl).getDb();
				addDependentModule(new DatabaseModule().addDatabases(db).getModule());
			}
		}

		public LockingModuleInner() {
			super();
			addInitializable(new BaseInitializable() {
				
				@Override
				protected void shutdown() {
					if (semaphoreImpl instanceof GlobalSemaphoreImpl &&
							((GlobalSemaphoreImpl) semaphoreImpl).getDb() != null) {
						BaseDatabaseType db = ((GlobalSemaphoreImpl) semaphoreImpl).getDb();
						if (db instanceof HsqlDatabaseType) {
							try {
								shutdownMemTable(db);
							} catch (SQLException e) {
								throw new ShutdownException(e);
							}
						}
					}
				}
				
				@Override
				protected void initialize() {
					if (semaphoreImpl instanceof GlobalSemaphoreImpl &&
							((GlobalSemaphoreImpl) semaphoreImpl).getDb() != null) {
						BaseDatabaseType db = ((GlobalSemaphoreImpl) semaphoreImpl).getDb();
						if (db == null || !db.isShared()) { 
							throw new InitializationException("Global locking requires shared database");
						}
						
						GlobalSemaphoreDAO.getInstance().setDbEnum(db);
						
						if (db instanceof HsqlDatabaseType) {
							try {
								setupMemTable(db);
							} catch (SQLException e) {
								throw new InitializationException(e);
							}
						}
					}
				}
			});
		}
		
		void setupMemTable(BaseDatabaseType db) throws SQLException {
			ConnectionHelper.executeUpdate(db, "drop table GLOBAL_SEMAPHORE if exists");
			ConnectionHelper.executeUpdate(db, 
				"Create table GLOBAL_SEMAPHORE(" +
					"semaphore_id        INTEGER," +
					"semaphore_key       VARCHAR(128)," +
					"CREATION_DATE       TIMESTAMP," +
					"last_modified_date  TIMESTAMP)");
			ConnectionHelper.executeUpdate(db, "drop sequence GLOBAL_SEMAPHORE_ID_SEQ if exists");
			ConnectionHelper.executeUpdate(db, "create sequence GLOBAL_SEMAPHORE_ID_SEQ start with 1 increment by 1");
		}
		
		void shutdownMemTable(BaseDatabaseType db) throws SQLException {
			ConnectionHelper.executeUpdate(db, "drop table GLOBAL_SEMAPHORE if exists");
			ConnectionHelper.executeUpdate(db, "drop sequence GLOBAL_SEMAPHORE_ID_SEQ if exists");
		}
	
	}
	
	

}
