package org.lightj.locking.dal;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import org.lightj.dal.AbstractDAO;
import org.lightj.dal.BaseDatabaseType;
import org.lightj.dal.BaseSequenceEnum;
import org.lightj.dal.ConnectionHelper;
import org.lightj.dal.DataAccessException;
import org.lightj.dal.Query;


public class GlobalSemaphoreDAO extends AbstractDAO<GlobalSemaphoreDO> {

	private static GlobalSemaphoreDAO me = null;

	public static synchronized final GlobalSemaphoreDAO getInstance() {
		if (me == null) {
			me = new GlobalSemaphoreDAO();
		}
		return me;
	}

	private GlobalSemaphoreDAO() {
		super();
		register(GlobalSemaphoreDO.class, GlobalSemaphoreDO.TABLENAME, null, BaseSequenceEnum.SEQ_GLOBALSEMAPHORE,
				new String[] { "semaphore_id","semaphore_key", "CREATION_DATE", "last_modified_date" },
				new String[] { "semaphoreId", "semaphoreKey", "creationDate", "lastModifiedDate" });
	}

	/**
	 * lock
	 * @param semaphoreDO
	 * @throws DataAccessException
	 * @throws SQLException
	 */
	public void lockSemaphore(BaseDatabaseType lockDb, final GlobalSemaphoreDO semaphoreDO) throws DataAccessException, SQLException{
		//Connection conn = ConnectionHelper.getConnection(this.getDbEnum());
		int numberOfRowsLocked = aquireSemaphore (lockDb, semaphoreDO);
		log.debug(" numberOfRowsLocked returned when trying to lock the semaphore : " + numberOfRowsLocked);
		
		if (numberOfRowsLocked == 0 ) {
			Connection con = ConnectionHelper.getConnection(lockDb);
			// SYNC Condition: all the threads will try to commit at the same time.
			// which means that if there are 4 threads , 1 commit will succeed and 3 will fail, 
			// in that case we will want to go aquire the lock again, so basically catch the exception 
			// and try to aquire the lock until we get it.
			try{
				super.insert(semaphoreDO); 
				con.commit();
				//commitTr(this.getDbEnum());
				//
			}catch (Exception e){
				log.debug("Insert commit failed.This might be expected, this is the first time we are locking a semaphore on this object key with many threads");
				//commit failed lets rollback right here the next call to acquire semaphore is going to create a new connection.
				//rollback transaction manually.
				//ConnectionHelper.rollbackTr(this.getDbEnum());
				con.rollback();
			} finally {
				ConnectionHelper.cleanupDBResources(null, null, con);
			}
			
			numberOfRowsLocked = aquireSemaphore(lockDb, semaphoreDO);
			
			log.debug("Trying to acquire the lock for the second time numberOfRowsLocked returned when trying to lock the semaphore : " + numberOfRowsLocked);
			if (numberOfRowsLocked == 0 ){
				throw new DataAccessException ("I was not able to aquire the semaphore");
			}
		}
	}
	
	/**
	 * Select for update happens right here.
	 * */
	@SuppressWarnings("rawtypes")
	private int aquireSemaphore(BaseDatabaseType lockDb, GlobalSemaphoreDO semaphoreDO) throws DataAccessException{

		Query query = new Query();
		query.select("*");
		query.from(GlobalSemaphoreDO.TABLENAME, "l");
		query.or("l.semaphore_key","=",semaphoreDO.getSemaphoreKey());

		StringBuffer sql = new StringBuffer();
		if (query.isFullQuery()) {
			sql.append(query.toString());
		} 
		else  {
			sql.append("SELECT * FROM ").append(this.tableName).append(query.daoString());
		}
		sql.append(" ").append("FOR UPDATE");

		log.debug(this.getClass().getName() + ".search(...) is executing " + query.debugString());
		
		Connection con = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		final List args = query.getArgs();
		int counter = 0;
		try {
			con = ConnectionHelper.getConnection(lockDb);
			stmt = con.prepareStatement(sql.toString());
			for (int i = 1, len = args.size(); i <= len; i++) {
				stmt.setObject(i, args.get(i-1));
			}
			rs = stmt.executeQuery();
			if (rs.next()) 		counter++;
			
		}catch (SQLException e){
			throw new DataAccessException("Failed to acquire lock on the semaphore ");
		}finally{
			ConnectionHelper.cleanupDBResources(rs, stmt, con);
		}

		return counter;
	}
	
}
