package org.lightj.schedule;

import java.util.Date;

/**
 * request to schedule
 * @author biyu
 *
 */
public interface IScheduleRequest {
	
	/**
	 * set description
	 * @param description
	 */
	public void setDescription(String description);
	
	/**
	 * get description
	 * @return
	 */
	public String getDescription();
	
	/**
	 * which scheduler to use
	 * @return
	 */
	public String getSchedulerKey();
	
	/**
	 * which scheduler to use
	 * @param key
	 */
	public void setSchedulerKey(String key);

	/**
	 * Execute once at executionTime, if null, execute immediately
	 * @param executionTime
	 * @throws ScheduleException
	 */	
	public void executeAt(Date executionTime);
	
	/**
	 * Execute repeatly from startTime to endTime with interval, 
	 * at most repeat repeatCount times. This method round all times to second.
	 * @param startTime		start execution, immediately if null
	 * @param endTime		end execution
	 * @param repeatCount	max repeat times
	 * @param intervalSec	repeat interval in second
	 * @throws ScheduleException
	 */
	public void executeRecur(Date startTime, Date endTime, int repeatCount, long intervalSec);
	
	/**
	 * Execute based on a cron expression. This method round all times to second.
	 * @param startTime		start time when cron expression is valid
	 * @param endTime		end time when cron expression is valid
	 * @param cronExpression
	 * @throws ScheduleException
	 */ 
	public void executeCron(Date startTime, Date endTime, String cronExpression);
	
	/**
	 * pass schedule request to real schedule
	 * @param schedule
	 * @throws ScheduleException
	 */
	public void schedule(ISchedule schedule) throws ScheduleException;
	
}
