package org.lightj.schedule;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Date;
import java.util.UUID;

import org.lightj.util.StringUtil;

/**
 * a request to schedule
 * @author biyu
 *
 */
public class ScheduleRequest implements IScheduleRequest {
	
	static enum ExecType {
		AT,
		CRON,
		RECUR,
	}
	
	/** key of scheduler */
	private String schedulerKey;
	/** exec type */
	private ExecType type;
	/** data */
	private Object[] data;
	/** description */
	private String desc;
	
	public ScheduleRequest(String schedulerKey) {
		this.schedulerKey = schedulerKey;
	}

	@Override
	public void executeAt(Date executionTime) {
		type = ExecType.AT;
		data = new Object[] {executionTime};
	}

	@Override
	public void executeCron(Date startTime, Date endTime, String cronExpression) {
		type = ExecType.CRON;
		data = new Object[] {startTime, endTime, cronExpression};
	}

	@Override
	public void executeRecur(Date startTime, Date endTime, int repeatCount,
			long intervalSec) {
		type = ExecType.RECUR;
		data = new Object[] {startTime, endTime, repeatCount, intervalSec};
	}

	@Override
	public String getSchedulerKey() {
		return schedulerKey;
	}

	@Override
	public void schedule(ISchedule schedule) throws ScheduleException {
		Method m = null;
		try {
			switch(type) {
			case AT:
				m = schedule.getClass().getMethod("executeAt", new Class[] {Date.class});
				break;
			case CRON:
				m = schedule.getClass().getMethod("executeCron", new Class[] {Date.class, Date.class, String.class});
				break;
			case RECUR:
				m = schedule.getClass().getMethod("executeRecur", new Class[] {Date.class, Date.class, int.class, long.class});
				break;
			default:
				throw new ScheduleException("No schedule set");
			}
			m.invoke(schedule, data);
		} catch (SecurityException e) {
			throw new ScheduleException(e);
		} catch (NoSuchMethodException e) {
			throw new ScheduleException(e);
		} catch (IllegalArgumentException e) {
			throw new ScheduleException(e);
		} catch (IllegalAccessException e) {
			throw new ScheduleException(e);
		} catch (InvocationTargetException e) {
			throw new ScheduleException(e);
		}
	}

	@Override
	public void setSchedulerKey(String key) {
		this.schedulerKey = key;
	}

	@Override
	public String getDescription() {
		return StringUtil.isNullOrEmpty(desc) ? UUID.randomUUID().toString() : desc;
	}

	@Override
	public void setDescription(String description) {
		this.desc = description;
	}

}
