package org.lightj.session.dal;

import java.util.List;

import org.lightj.dal.BaseDatabaseType;
import org.lightj.dal.DataAccessException;
import org.lightj.dal.DataAccessRuntimeException;
import org.lightj.dal.Query;

/**
 * session data manager interface
 * @author biyu
 *
 * @param <T>
 */
public interface ISessionDataManager<T extends ISessionData> {

	/**
	 * create new instance of {@link ISessionData} 
	 * @return
	 * @throws DataAccessRuntimeException
	 */
	public T newInstance() throws DataAccessRuntimeException;
	
	/**
	 * save {@link ISessionData}
	 * @param data
	 * @throws DataAccessException
	 */
	public void save(T data) throws DataAccessException;
	
	/**
	 * delete {@link ISessionData}
	 * @param data
	 * @throws DataAccessException
	 */
	public void delete(T data) throws DataAccessException;
	
	/**
	 * find {@link ISessionData} by its id
	 * @param id
	 * @param fromCache
	 * @return
	 * @throws DataAccessException
	 */
	public T findById(long id, boolean fromCache) throws DataAccessException;
	
	/**
	 * find {@link ISessionData}s by query
	 * @param query
	 * @return
	 * @throws DataAccessException
	 */
	public List<T> search(Query query) throws DataAccessException;
	
	/**
	 * associate with a datastore 
	 * @param dbEnum
	 */
	public void setDbEnum(BaseDatabaseType dbEnum);

}
