package org.lightj.session.step;

import java.util.ArrayList;
import java.util.List;

import org.lightj.session.FlowContext;
import org.lightj.session.FlowExecutionException;
import org.lightj.task.ExecuteOption;
import org.lightj.task.ITaskListener;
import org.lightj.task.Task;
import org.lightj.task.TaskExecutionException;
import org.lightj.task.TaskMonitor;
import org.lightj.task.TaskRunner;

import edu.emory.mathcs.backport.java.util.Arrays;


/**
 * a flow step to run a task via a task runner, async
 * 
 * @author biyu
 *
 */
@SuppressWarnings({"rawtypes", "unchecked"})
public class AsyncTaskStepExecution<T extends FlowContext> extends StepExecution<T> {
	
	/**
	 * task to be runned
	 */
	protected List<Task> tasks = new ArrayList<Task>();

	/**
	 * task runner
	 */
	protected TaskRunner taskRunner;
	
	/**
	 * task monitor
	 */
	protected TaskMonitor taskMonitor;
	
	/**
	 * execute options
	 */
	protected ExecuteOption execOptions;
	
	/**
	 * task listener
	 */
	protected ITaskListener taskListener;
	
	
	/**
	 * constructor
	 * @param task
	 * @param taskRunner
	 */
	public AsyncTaskStepExecution(TaskRunner runner, ITaskListener taskListener) {
		this.taskRunner = runner;
		this.taskListener = taskListener;
	}
	
	/**
	 * constructor
	 * @param task
	 * @param runner
	 * @param taskMonitor
	 */
	public AsyncTaskStepExecution(TaskRunner runner, ITaskListener taskListener, TaskMonitor taskMonitor) {
		this.taskRunner = runner;
		this.taskListener = taskListener;
		this.taskMonitor = taskMonitor;
	}

	/**
	 * submit the task to be run by task runner
	 */
	@Override
	public StepTransition execute() throws FlowExecutionException {
		if (taskRunner == null) throw new FlowExecutionException("task runner cannot be empty");
		if (taskListener == null) throw new FlowExecutionException("task listener cannot be empty");
		if (tasks.isEmpty()) {
			return defResult;
		}
		int failOnSubmissionCount = 0;
		for (Task task : tasks) {
			try {
				if (taskMonitor == null) {
					taskRunner.runASyncTask(taskListener, task, execOptions);
				}
				else {
					taskRunner.runASyncPollTask(taskListener, task, taskMonitor);
				}
			} 
			catch (TaskExecutionException e) {
				failOnSubmissionCount++;
				
			}
		}
		if (failOnSubmissionCount == tasks.size()) {
			throw new FlowExecutionException("all tasks failed on submission");
		}
		return StepTransition.CALLBACK;
	}

	public List<Task> getTasks() {
		return tasks;
	}

	/** add flows */
	public void addTasks(Task... tasks) {
		this.tasks.addAll(Arrays.asList(tasks));
	}

	public TaskRunner getTaskRunner() {
		return taskRunner;
	}

	public void setTaskRunner(TaskRunner taskRunner) {
		this.taskRunner = taskRunner;
	}

	public TaskMonitor getTaskMonitor() {
		return taskMonitor;
	}

	public void setTaskMonitor(TaskMonitor taskMonitor) {
		this.taskMonitor = taskMonitor;
	}

	public ExecuteOption getExecOptions() {
		return execOptions;
	}

	public void setExecOptions(ExecuteOption execOptions) {
		this.execOptions = execOptions;
	}

}
