package org.lightj.session.step;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;

import org.lightj.util.StringUtil;

import edu.emory.mathcs.backport.java.util.Arrays;


/**
 * retry enclosure
 * @author binyu
 *
 */
public class RetryEnclosure implements InvocationHandler {
	
	@SuppressWarnings("unchecked")
	static Set<String> methods = new HashSet<String>(Arrays.asList(new String[] {"onExecutionError","execute","onResult","onResultError"}));
	
	private Object realImpl;
	private StepTransition transition;
	private int maxRetry = 0;
	private long delay = 0;
	private AtomicInteger retry = new AtomicInteger(0);

	public RetryEnclosure(Object impl, StepTransition transition, int maxRetry, long delay) {
		this.realImpl = impl;
		this.transition = transition;
		this.maxRetry = maxRetry;
		this.delay = delay;
	}

	@Override
	public Object invoke(Object proxy, final Method method, final Object[] args)
			throws Throwable {
		if (Object.class == method.getDeclaringClass()) {
			String name = method.getName();
			if ("equals".equals(name)) {
				return proxy == args[0];
			} else if ("hashCode".equals(name)) {
				return System.identityHashCode(proxy);
			} else if ("toString".equals(name)) {
				return proxy.getClass().getName() + "@"
						+ Integer.toHexString(System.identityHashCode(proxy))
						+ ", with InvocationHandler " + this;
			} else {
				throw new IllegalStateException(String.valueOf(method));
			}
		}
		else if (methods.contains(method.getName())) {
//			System.out.println("fire method: " + method.getName());
			Object rst = method.invoke(realImpl, args);
			if (rst != null && rst instanceof StepTransition) {
				StepTransition cur = (StepTransition) rst;
				if (cur.getActionStatus() == transition.getActionStatus() && 
						cur.getResultStatus() == transition.getResultStatus() &&
						StringUtil.equalIgnoreCase(cur.getNextStep(), transition.getNextStep())) {
					if (retry.incrementAndGet() <= maxRetry) {
//						System.out.println("transition matched, retrying with count: " + retry.get());
						if (realImpl instanceof IFlowStep) {
							String stepName = ((IFlowStep) realImpl).getStepName();
							if (delay > 0) {
								Thread.sleep(delay);
							}
							return StepTransition.runToStep(stepName).log("Retry step " + stepName + " with retry count " + retry.get());
						}
					}
					else {
//						System.out.println("transition matched over retry limit: " + retry.get());
					}
				}
			}
			return rst;
		}
		else {
			return method.invoke(realImpl, args);
		}
		
	}	
	
	/**
	 * make a step retry for maxRetry time on matching condition
	 * @param condition
	 * @param maxRetry
	 * @param delay
	 * @param step
	 * @return
	 */
	public static IFlowStep retry(StepTransition condition, int maxRetry, long delay, IFlowStep step) {
		   return (IFlowStep) Proxy.newProxyInstance(IFlowStep.class.getClassLoader(),
                   new Class<?>[] {IFlowStep.class},
                   new RetryEnclosure(step, condition, maxRetry, delay));
	}

}
