package org.lightj.session.step;

import org.lightj.session.FlowContext;
import org.lightj.session.FlowExecutionException;

/**
 * An execution aspect of a flow step for error handling
 * 
 * @author biyu
 *
 */
public abstract class StepErrorHandler<T extends FlowContext> extends StepExecution<T> {
	
	public StepErrorHandler(StepTransition transition) {
		super(transition);
	}

	/**
	 * trigger error 
	 */
	protected Throwable t;

	@Override
	public final StepTransition execute() throws FlowExecutionException {
		return executeOnError(t);
	}
	
	/**
	 * set triggering error
	 * @param t
	 */
	public final void setError(Throwable t) {
		this.t = t;
	}
	
	/**
	 * do the work when error happens
	 * @param t
	 * @return
	 */
	protected abstract StepTransition executeOnError(Throwable t);

}
