package org.lightj.task;

import java.util.concurrent.Delayed;
import java.util.concurrent.TimeUnit;

/**
 * monitor event with {@link Delayed} interface
 * 
 * @author biyu
 *
 */
public final class MonitorEvent<T extends Task> implements Delayed {
	
	/** handle monitoring */
	protected TaskMonitor<T> taskMonitor;
	
	/** listener */
	protected ITaskListener taskListener;
	
	/** original task */
	protected T task;
	
	/** monitor option */
	protected MonitorOption monitorOption;

	/** constructor */
	public MonitorEvent(TaskMonitor<T> taskMonitor, ITaskListener taskListener, T task) 
	{
		if (taskMonitor == null || taskMonitor.getMonitorOption() == null) throw new IllegalArgumentException("Task monitor or monitor option can not be null");
		if (task == null) throw new IllegalArgumentException("Task can not be null");

		this.task = task;
		this.taskMonitor = taskMonitor;
		this.taskListener = taskListener;
		this.monitorOption = this.taskMonitor.getMonitorOption();
	}
	
	/** listener */
	public ITaskListener getTaskListener() {
		return taskListener;
	}

	/** task monitor */
	public TaskMonitor<T> getTaskMonitor() {
		return taskMonitor;
	}

	/** original task */
	public T getTask() {
		return task;
	}

	/** wether this event has been timed out */
	public synchronized boolean isTimedOut() {
		return System.currentTimeMillis() >= monitorOption.getTimeOutAt();
	}

	/** schedule next run time */
	public synchronized void scheduleNextRun() {
		monitorOption.scheduleNextRun();
	}

	/** name */
	public String getName() {
		return task!=null ? task.getTaskId() : null;
	}
	
	/** string description */
	public String toString() {
		return getName();
	}

	////////////// Delayed interface ////////////
	
	public long getDelay(TimeUnit unit) {
		long delay = Math.min(monitorOption.getNextRunAt(), monitorOption.getTimeOutAt()) - System.currentTimeMillis();
		return unit.convert(delay, TimeUnit.MILLISECONDS);
	}

	@SuppressWarnings("rawtypes")
	public int compareTo(Delayed o) {
		return (Long.valueOf(monitorOption.getNextRunAt())).compareTo(Long.valueOf(((MonitorEvent) o).monitorOption.getNextRunAt()));
	}

}
