package org.lightj.task;

/**
 * monitor a task progress
 * @author biyu
 *
 */
public abstract class TaskMonitor<T extends Task> {
	
	/**
	 * monitor option
	 */
	protected MonitorOption monitorOption;
	
	public TaskMonitor(MonitorOption monitorOption) {
		this.monitorOption = monitorOption;
	}

	/**
	 * schedule poll
	 * @param taskListener
	 * @param task
	 */
	public void scheduleMonitor(ITaskListener taskListener, T task) {
		MonitorEvent<T> event = new MonitorEvent<T>(this, taskListener, task);
		MonitorRunner.getInstance().addToQueue(event);
	}

	public MonitorOption getMonitorOption() {
		return monitorOption;
	}

	public void setMonitorOption(MonitorOption monitorOption) {
		this.monitorOption = monitorOption;
	}
	
	/** 
	 * handle cancellation, framework will callback tasklistener for cancel event, 
	 * this is to handle any additional clean up may be needed  
	 * @param event
	 * @param reason
	 * @throws TaskExecutionException
	 */
	public abstract void handleCancel(MonitorEvent<T> event, String reason) throws TaskExecutionException;

	/**
	 * handle timeout, framework will callback tasklistener for timeout event,
	 * this is to handle any additional cleanup may needed
	 * handle timeout
	 * @param event
	 * @throws TaskExecutionException
	 */
	public abstract void handleTimeout(MonitorEvent<T> event) throws TaskExecutionException;

	/** 
	 * monitor 
	 *@return boolean if the event need to be rescheduled 
	 */
	public abstract boolean monitorTask(MonitorEvent<T> event) throws TaskExecutionException;


}
