package org.lightj.task;

import java.io.Serializable;
import java.util.HashMap;

/**
 * @author biyu
 * 
 */ 
public class TaskResult implements Serializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -7755292862995731954L;

	/** task definition associated with the result */
	protected Task task;
	
	/** result status */
	protected TaskResultEnum status = TaskResultEnum.Unknown;
	
	/** result msg */
	protected String msg;
	
	/** properties */
	protected HashMap<String, String> meta = new HashMap<String, String>();
	
	/** constructor */
	@SuppressWarnings("unused")
	private TaskResult() {}

	/**
	 * constructor
	 * @param task
	 * @param status
	 * @param msg
	 */
	public TaskResult(Task task, TaskResultEnum status, String msg) {
		this.task = task;
		this.status = status;
		this.msg = msg;
	}

	/** get task definition */
	public Task getTask() {
		return task;
	}
	
	/** set task definition */
	public void setTask(Task definition) {
		task = definition;
	}
	
	public HashMap<String, String> getMeta() {
		return meta;
	}

	public void setMeta(HashMap<String, String> meta) {
		this.meta = meta;
	}

	public String getMsg() {
		return msg;
	}

	public void setMsg(String msg) {
		this.msg = msg;
	}

	public TaskResultEnum getStatus() {
		return status;
	}

	public void setStatus(TaskResultEnum string) {
		status = string;
	}

	/** create a immutable TaskResult */
	public static final TaskResult createLocal(Task task, TaskResultEnum status, String msg) {
		return new TaskResult(task, status, msg);
	}
	public boolean isSuccess() {
		return getStatus() == TaskResultEnum.Success;
	}
	public boolean isFailed() {
		return getStatus() == TaskResultEnum.Failed;
	}
	public boolean isTimedOut() {
		return getStatus() == TaskResultEnum.Timeout;
	}
	public boolean isCancelled() {
		return getStatus() == TaskResultEnum.Canceled;
	}
	public boolean isMoreSevere(TaskResult another) {
		return this.status.getSeverity() > another.getStatus().getSeverity();
	}

	/////////////////// proxy to task definition ////////////////
	public String getTaskName() {
		return task.getName();
	}
	public String getTaskId() {
		return task.getTaskId();
	}
}
