package org.lightj.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


/**
 * This class represents IPv4 address
 *
 * @author Jagath Vijayan Janakiraman
 * @created 04/30/2008;
 */
public class IPv4Address implements Comparable<IPv4Address> {

	private long ip;

	private String ipDotted;

	/**
	 * Input Format (IPv4 - Dotted Decimal): A.B.C.D
	 * Where,
	 * 0 <= A/B/C/D <= 255
	 *
	 * Ex: 10.26.200.0
	 */
	public IPv4Address(String ipAddress) throws InvalidIPv4AddressException {
		if (isValid(ipAddress)) {
			this.ipDotted = ipAddress;
		} else {
			throw new InvalidIPv4AddressException(ipAddress);
		}
	}

	public String toString() {
		return this.ipDotted;
	}

	public long getIPv4Number() {
		return this.ip;
	}

	private boolean isValid(String ipAddress) {
		boolean isValid = false;
		if (ipAddress != null && ipAddress.length() > 0) {
			String[] octets = ipAddress.split("\\.");
			if (octets != null && octets.length == 4) {
				try {
					long ip = 0;
					for (int i = 0; i < 4; i++) {
						String octet = octets[i];
						int j = Integer.parseInt(octet);
						if (j >= 0 && j < 256) {
							ip = ip + (j * (long)(Math.pow(256, (3 - i))));
						} else {
							break;
						}
						isValid = (i == 3);
					}
					if (isValid) {
						this.ip = ip;
					}
				} catch (NumberFormatException nfe) {
					isValid = false;
				}
			}
		}
		return isValid;
	}

	public boolean equals(Object o) {
		boolean result = false;
		if (o instanceof IPv4Address) {
			IPv4Address address = (IPv4Address) o;
			result = (this.getIPv4Number() == address.getIPv4Number());
		}
		return result;
	}
	
	public int compareTo(IPv4Address address) {
		int i = 0;
		if (this.getIPv4Number() < address.getIPv4Number()) {
			i = -1;
		} else if (this.getIPv4Number() > address.getIPv4Number()) {
			i = 1;
		}
		return i;
	}
	
	public static boolean isContiguous(String[] ipAddresses) 
			throws InvalidIPv4AddressException {
		
		boolean result = false;
		if (ipAddresses != null && ipAddresses.length > 0) {
			List<IPv4Address> ipv4Addresses = new ArrayList<IPv4Address>();
			for (String ipAddr : ipAddresses) {
				IPv4Address ipv4address = new IPv4Address(ipAddr);
				ipv4Addresses.add(ipv4address);
			}
			Collections.sort(ipv4Addresses);
			int i = 0;
			for (IPv4Address current : ipv4Addresses) {
				if (i < (ipv4Addresses.size() - 1)) {
					IPv4Address next = ipv4Addresses.get(++i);
					if (next.getIPv4Number() == (current.getIPv4Number() + 1)) {
						continue;
					} else {
						break;
					}
				} else {
					result = true;
					break;
				}
			}
		}
		return result;
	}
	
}