package org.lightj.util;



/**
 * This class represents CIDR format for IPv4 address
 *
 * @author Jagath Vijayan Janakiraman
 * @created 04/30/2008;
 */
public class IPv4CIDR {

	private String cidr;

	private IPv4Address network;

	private int prefix;

	/**
	 * Input Format (CIDR): A.B.C.D/N
	 * Where,
	 * 0 <= A/B/C/D <= 255
	 * 0 <= N <= 32
	 *
	 * Ex: 10.26.200.0/24
	 */
	public IPv4CIDR(String cidr) throws InvalidIPv4CIDRException {
		if (isValid(cidr)) {
			this.cidr = cidr;
		} else {
			throw new InvalidIPv4CIDRException(cidr);
		}
	}

	public String toString() {
		return this.cidr;
	}

	public int getPrefixLength() {
		return this.prefix;
	}

	private boolean isValid(String cidr) {
		boolean isValid = false;
		if (cidr != null && cidr.length() > 0) {
			String parts[] = cidr.split("\\/");
			if (parts != null && parts.length == 2) {
				try {
					String network = parts[0];
					IPv4Address address = new IPv4Address(network);
					int prefix = Integer.parseInt(parts[1]);
					if (prefix >= 0 && prefix <= 32) {
						this.network = address;
						this.prefix = prefix;
						isValid = true;
					}
				} catch (InvalidIPv4AddressException iiae) {
					isValid = false;
				}
			}
		}
		return isValid;
	}

	public boolean isIPInNetwork(String address)
			throws InvalidIPv4AddressException {

		boolean inNetwork = false;
		if (address != null && address.length() > 0) {
			IPv4Address ipAddress = new IPv4Address(address);
			if (ipAddress != null && this.network != null) {
				StringBuffer leadingOnes = new StringBuffer();
				for (int i = 1; i <= prefix; i++) {
					leadingOnes.append(1);
				}
				long mask = 0;
				if (leadingOnes.toString().length() > 0) {
					mask = Long.parseLong(leadingOnes.toString(), 2);
				}
				mask = (mask << (32 - prefix));
				long ip = ipAddress.getIPv4Number();
				long network = this.network.getIPv4Number();
				inNetwork = ((network & mask) == (ip & mask));
			}

		}
		return inNetwork;
	}

}
