package org.lightj.util;

import org.codehaus.janino.ClassBodyEvaluator;
import org.codehaus.janino.CompileException;
import org.codehaus.janino.ExpressionEvaluator;
import org.codehaus.janino.ScriptEvaluator;
import org.codehaus.janino.Parser.ParseException;
import org.codehaus.janino.Scanner.ScanException;

@SuppressWarnings("rawtypes")
public class JaninoCompiler {

	/**
	 * Method to compile expression
	 * @param expression
	 * @param expressionType
	 * @param parameterNames
	 * @param parameterTypes
	 * @param optionalDefaultImports 
	 * @return complied expression 
	 * @throws CompileException
	 * @throws ParseException
	 * @throws ScanException
	 */
	public static ExpressionEvaluator compileExpression(String[] optionalDefaultImports,String expression, Class expressionType,
	      String[] parameterNames, Class[] parameterTypes) throws CompileException, ParseException, ScanException {
		ExpressionEvaluator exprEvaluator = new ExpressionEvaluator();
		exprEvaluator.setThrownExceptions(new Class[] { CompileException.class });
		exprEvaluator.setExpressionType(expressionType);
		exprEvaluator.setParameters(parameterNames, parameterTypes);
		exprEvaluator.setDefaultImports(optionalDefaultImports);
		exprEvaluator.cook(expression);
		return exprEvaluator;
	}

	/**
	 * Method to compile script
	 * @param importSatements
	 * @param expression
	 * @param returnType
	 * @param parameterNames
	 * @param parameterTypes
	 * @return complied script 
	 * @throws CompileException
	 * @throws ParseException
	 * @throws ScanException
	 */
	public static ScriptEvaluator compileScript(String[] importSatements, String expression, Class returnType,
	      String[] parameterNames, Class[] parameterTypes) throws CompileException, ParseException, ScanException {
		ScriptEvaluator scriptEvaluator = new ScriptEvaluator();
		scriptEvaluator.setThrownExceptions(new Class[] { CompileException.class });
		 scriptEvaluator.setDefaultImports(importSatements);
		scriptEvaluator.setReturnType(returnType);
		scriptEvaluator.setParameters(parameterNames, parameterTypes);
		scriptEvaluator.cook(expression);
		return scriptEvaluator;
	}

	/**
	 * Method to compile whole class body
	 * @param classBody
	 * @return class definition
	 * @throws CompileException
	 * @throws ParseException
	 * @throws ScanException
	 */
	public static Class compileClassBody(String classBody) throws CompileException, ParseException, ScanException {
		// Compile the class body.
		Class classDef = new ClassBodyEvaluator(classBody).getClazz();
		return classDef;
	}
}
