-- global semaphore tables

create table if not exists GLOBAL_SEMAPHORE (
	semaphore_id	bigint auto_increment primary key,
	semaphore_key	varchar(256),
	CREATION_DATE	timestamp, 
	last_modified_date	timestamp
 );
 
create index gs_key_NDX on GLOBAL_SEMAPHORE (semaphore_key);

-- flow session tables

create table if not exists flow_session (
	flow_id		bigint auto_increment primary key, 
	creation_date	timestamp,
	end_date	timestamp, 
	flow_status	varchar(512),
	target		varchar(512),
	flow_type	integer, 
	parent_id	bigint references flow_session(flow_id),
	current_action	varchar(128), 
	next_action	varchar(128), 
	flow_state	varchar(64), 
	flow_result	varchar(64), 
	last_modified	timestamp, 
	run_by		varchar(256),
	requester	varchar(512)
);

create index SESCOP_PRNT_IDX on FLOW_SESSION (PARENT_ID);
create index FS_ACTIONSTATUS on FLOW_SESSION (FLOW_STATE);
create index FS_CD_IDX on FLOW_SESSION (CREATION_DATE);
create index FS_EI on FLOW_SESSION (END_DATE, FLOW_ID);
create index FS_ENDDATE on FLOW_SESSION (END_DATE);
create index FS_KEY on FLOW_SESSION (TARGET);
create index FS_TYPE_IDX on FLOW_SESSION (FLOW_TYPE);


create table if not exists flow_session_meta (
	flow_meta_id	bigint auto_increment primary key,
	flow_id		bigint references flow_session(flow_id),
	name		varchar(512),
	str_value	varchar(2000),
	blob_value	blob
);

CREATE INDEX FSM_SSNID_IDX ON FLOW_SESSION_META (FLOW_ID);

create table if not exists flow_step_log (
	flow_step_id	bigint auto_increment primary key,
	flow_id		bigint references flow_session(flow_id),
	step_name	varchar(512),
	creation_time	timestamp,
	result		varchar(512),
	details		varchar(2000)
);


-- quartz tables

DROP TABLE IF EXISTS LJ_JOB_LISTENERS;
DROP TABLE IF EXISTS LJ_TRIGGER_LISTENERS;
DROP TABLE IF EXISTS LJ_FIRED_TRIGGERS;
DROP TABLE IF EXISTS LJ_PAUSED_TRIGGER_GRPS;
DROP TABLE IF EXISTS LJ_SCHEDULER_STATE;
DROP TABLE IF EXISTS LJ_LOCKS;
DROP TABLE IF EXISTS LJ_SIMPLE_TRIGGERS;
DROP TABLE IF EXISTS LJ_CRON_TRIGGERS;
DROP TABLE IF EXISTS LJ_BLOB_TRIGGERS;
DROP TABLE IF EXISTS LJ_TRIGGERS;
DROP TABLE IF EXISTS LJ_JOB_DETAILS;
DROP TABLE IF EXISTS LJ_CALENDARS;


CREATE TABLE LJ_JOB_DETAILS
  (
    JOB_NAME  VARCHAR(200) NOT NULL,
    JOB_GROUP VARCHAR(200) NOT NULL,
    DESCRIPTION VARCHAR(250) NULL,
    JOB_CLASS_NAME   VARCHAR(250) NOT NULL,
    IS_DURABLE VARCHAR(1) NOT NULL,
    IS_VOLATILE VARCHAR(1) NOT NULL,
    IS_STATEFUL VARCHAR(1) NOT NULL,
    REQUESTS_RECOVERY VARCHAR(1) NOT NULL,
    JOB_DATA BLOB NULL,
    PRIMARY KEY (JOB_NAME,JOB_GROUP)
);

CREATE TABLE LJ_JOB_LISTENERS
  (
    JOB_NAME  VARCHAR(200) NOT NULL,
    JOB_GROUP VARCHAR(200) NOT NULL,
    JOB_LISTENER VARCHAR(200) NOT NULL,
    PRIMARY KEY (JOB_NAME,JOB_GROUP,JOB_LISTENER),
    FOREIGN KEY (JOB_NAME,JOB_GROUP)
        REFERENCES LJ_JOB_DETAILS(JOB_NAME,JOB_GROUP)
);

CREATE TABLE LJ_TRIGGERS
  (
    TRIGGER_NAME VARCHAR(200) NOT NULL,
    TRIGGER_GROUP VARCHAR(200) NOT NULL,
    JOB_NAME  VARCHAR(200) NOT NULL,
    JOB_GROUP VARCHAR(200) NOT NULL,
    IS_VOLATILE VARCHAR(1) NOT NULL,
    DESCRIPTION VARCHAR(250) NULL,
    NEXT_FIRE_TIME BIGINT(13) NULL,
    PREV_FIRE_TIME BIGINT(13) NULL,
    PRIORITY INTEGER NULL,
    TRIGGER_STATE VARCHAR(16) NOT NULL,
    TRIGGER_TYPE VARCHAR(8) NOT NULL,
    START_TIME BIGINT(13) NOT NULL,
    END_TIME BIGINT(13) NULL,
    CALENDAR_NAME VARCHAR(200) NULL,
    MISFIRE_INSTR SMALLINT(2) NULL,
    JOB_DATA BLOB NULL,
    PRIMARY KEY (TRIGGER_NAME,TRIGGER_GROUP),
    FOREIGN KEY (JOB_NAME,JOB_GROUP)
        REFERENCES LJ_JOB_DETAILS(JOB_NAME,JOB_GROUP)
);

CREATE TABLE LJ_SIMPLE_TRIGGERS
  (
    TRIGGER_NAME VARCHAR(200) NOT NULL,
    TRIGGER_GROUP VARCHAR(200) NOT NULL,
    REPEAT_COUNT BIGINT(7) NOT NULL,
    REPEAT_INTERVAL BIGINT(12) NOT NULL,
    TIMES_TRIGGERED BIGINT(10) NOT NULL,
    PRIMARY KEY (TRIGGER_NAME,TRIGGER_GROUP),
    FOREIGN KEY (TRIGGER_NAME,TRIGGER_GROUP)
        REFERENCES LJ_TRIGGERS(TRIGGER_NAME,TRIGGER_GROUP)
);

CREATE TABLE LJ_CRON_TRIGGERS
  (
    TRIGGER_NAME VARCHAR(200) NOT NULL,
    TRIGGER_GROUP VARCHAR(200) NOT NULL,
    CRON_EXPRESSION VARCHAR(200) NOT NULL,
    TIME_ZONE_ID VARCHAR(80),
    PRIMARY KEY (TRIGGER_NAME,TRIGGER_GROUP),
    FOREIGN KEY (TRIGGER_NAME,TRIGGER_GROUP)
        REFERENCES LJ_TRIGGERS(TRIGGER_NAME,TRIGGER_GROUP)
);

CREATE TABLE LJ_BLOB_TRIGGERS
  (
    TRIGGER_NAME VARCHAR(200) NOT NULL,
    TRIGGER_GROUP VARCHAR(200) NOT NULL,
    BLOB_DATA BLOB NULL,
    PRIMARY KEY (TRIGGER_NAME,TRIGGER_GROUP),
    FOREIGN KEY (TRIGGER_NAME,TRIGGER_GROUP)
        REFERENCES LJ_TRIGGERS(TRIGGER_NAME,TRIGGER_GROUP)
);

CREATE TABLE LJ_TRIGGER_LISTENERS
  (
    TRIGGER_NAME  VARCHAR(200) NOT NULL,
    TRIGGER_GROUP VARCHAR(200) NOT NULL,
    TRIGGER_LISTENER VARCHAR(200) NOT NULL,
    PRIMARY KEY (TRIGGER_NAME,TRIGGER_GROUP,TRIGGER_LISTENER),
    FOREIGN KEY (TRIGGER_NAME,TRIGGER_GROUP)
        REFERENCES LJ_TRIGGERS(TRIGGER_NAME,TRIGGER_GROUP)
);


CREATE TABLE LJ_CALENDARS
  (
    CALENDAR_NAME  VARCHAR(200) NOT NULL,
    CALENDAR BLOB NOT NULL,
    PRIMARY KEY (CALENDAR_NAME)
);



CREATE TABLE LJ_PAUSED_TRIGGER_GRPS
  (
    TRIGGER_GROUP  VARCHAR(200) NOT NULL, 
    PRIMARY KEY (TRIGGER_GROUP)
);

CREATE TABLE LJ_FIRED_TRIGGERS
  (
    ENTRY_ID VARCHAR(95) NOT NULL,
    TRIGGER_NAME VARCHAR(200) NOT NULL,
    TRIGGER_GROUP VARCHAR(200) NOT NULL,
    IS_VOLATILE VARCHAR(1) NOT NULL,
    INSTANCE_NAME VARCHAR(200) NOT NULL,
    FIRED_TIME BIGINT(13) NOT NULL,
    PRIORITY INTEGER NOT NULL,
    STATE VARCHAR(16) NOT NULL,
    JOB_NAME VARCHAR(200) NULL,
    JOB_GROUP VARCHAR(200) NULL,
    IS_STATEFUL VARCHAR(1) NULL,
    REQUESTS_RECOVERY VARCHAR(1) NULL,
    PRIMARY KEY (ENTRY_ID)
);

CREATE TABLE LJ_SCHEDULER_STATE
  (
    INSTANCE_NAME VARCHAR(200) NOT NULL,
    LAST_CHECKIN_TIME BIGINT(13) NOT NULL,
    CHECKIN_INTERVAL BIGINT(13) NOT NULL,
    PRIMARY KEY (INSTANCE_NAME)
);

CREATE TABLE LJ_LOCKS
  (
    LOCK_NAME  VARCHAR(40) NOT NULL, 
    PRIMARY KEY (LOCK_NAME)
);


INSERT INTO LJ_LOCKS values('TRIGGER_ACCESS');
INSERT INTO LJ_LOCKS values('JOB_ACCESS');
INSERT INTO LJ_LOCKS values('CALENDAR_ACCESS');
INSERT INTO LJ_LOCKS values('STATE_ACCESS');
INSERT INTO LJ_LOCKS values('MISFIRE_ACCESS');


commit;

