-- global semaphore tables
drop sequence GLOBAL_SEMAPHORE_ID_SEQ;
create sequence GLOBAL_SEMAPHORE_ID_SEQ start with 10000 increment by 1;

drop table GLOBAL_SEMAPHORE;
create table GLOBAL_SEMAPHORE (
	semaphore_id	number primary key,
	semaphore_key	varchar2(256),
	CREATION_DATE	timestamp, 
	last_modified_date	timestamp
 );
 
create index gs_key_NDX on GLOBAL_SEMAPHORE (semaphore_key);

-- flow session tables

drop sequence FLOW_ID_SEQ;
create sequence FLOW_ID_SEQ start with 10000 increment by 1;

drop table flow_session;
create table flow_session (
	flow_id		number primary key, 
	creation_date	timestamp,
	end_date	timestamp, 
	flow_status	varchar2(512),
	target		varchar2(512),
	flow_type	integer, 
	parent_id	number references flow_session(flow_id),
	current_action	varchar2(128), 
	next_action	varchar2(128), 
	flow_state	varchar2(64), 
	flow_result	varchar2(64), 
	last_modified	timestamp, 
	run_by		varchar2(256),
	requester	varchar2(512)
);

create index SESCOP_PRNT_IDX on FLOW_SESSION (PARENT_ID);
create index FS_ACTIONSTATUS on FLOW_SESSION (FLOW_STATE);
create index FS_CD_IDX on FLOW_SESSION (CREATION_DATE);
create index FS_EI on FLOW_SESSION (END_DATE, FLOW_ID);
create index FS_ENDDATE on FLOW_SESSION (END_DATE);
create index FS_KEY on FLOW_SESSION (TARGET);
create index FS_TYPE_IDX on FLOW_SESSION (FLOW_TYPE);


drop sequence FLOW_META_ID_SEQ;
create sequence FLOW_META_ID_SEQ start with 10000 increment by 1;

drop sequence flow_session_meta;
create table flow_session_meta (
	flow_meta_id	number primary key,
	flow_id		number references flow_session(flow_id),
	name		varchar2(512),
	str_value	varchar2(2000),
	blob_value	blob
);

CREATE INDEX FSM_SSNID_IDX ON FLOW_SESSION_META (FLOW_ID);

drop sequence FLOW_STEP_ID_SEQ;
create sequence FLOW_STEP_ID_SEQ start with 10000 increment by 1;

drop table flow_step_log
create table flow_step_log (
	flow_step_id	number primary key,
	flow_id		number references flow_session(flow_id),
	step_name	varchar2(512),
	creation_time	timestamp,
	result		varchar2(512),
	details		varchar2(2000)
);

-- quartz table for persistent scheduler
--
-- A hint submitted by a user: Oracle DB MUST be created as "shared" and the 
-- job_queue_processes parameter  must be greater than 2, otherwise a DB lock 
-- will happen.   However, these settings are pretty much standard after any
-- Oracle install, so most users need not worry about this.
--

--
-- A hint submitted by a user: Oracle DB MUST be created as "shared" and the 
-- job_queue_processes parameter  must be greater than 2, otherwise a DB lock 
-- will happen.   However, these settings are pretty much standard after any
-- Oracle install, so most users need not worry about this.
--
-- Many other users (including the primary author of Quartz) have had success
-- runing in dedicated mode, so only consider the above as a hint ;-)
--

delete from lj_job_listeners;
delete from lj_trigger_listeners;
delete from lj_fired_triggers;
delete from lj_simple_triggers;
delete from lj_cron_triggers;
delete from lj_blob_triggers;
delete from lj_triggers;
delete from lj_job_details;
delete from lj_calendars;
delete from lj_paused_trigger_grps;
delete from lj_locks;
delete from lj_scheduler_state;

drop table lj_calendars;
drop table lj_fired_triggers;
drop table lj_trigger_listeners;
drop table lj_blob_triggers;
drop table lj_cron_triggers;
drop table lj_simple_triggers;
drop table lj_triggers;
drop table lj_job_listeners;
drop table lj_job_details;
drop table lj_paused_trigger_grps;
drop table lj_locks;
drop table lj_scheduler_state;


CREATE TABLE lj_job_details
  (
    JOB_NAME  VARCHAR2(200) NOT NULL,
    JOB_GROUP VARCHAR2(200) NOT NULL,
    DESCRIPTION VARCHAR2(250) NULL,
    JOB_CLASS_NAME   VARCHAR2(250) NOT NULL, 
    IS_DURABLE VARCHAR2(1) NOT NULL,
    IS_VOLATILE VARCHAR2(1) NOT NULL,
    IS_STATEFUL VARCHAR2(1) NOT NULL,
    REQUESTS_RECOVERY VARCHAR2(1) NOT NULL,
    JOB_DATA BLOB NULL,
    PRIMARY KEY (JOB_NAME,JOB_GROUP)
);
CREATE TABLE lj_job_listeners
  (
    JOB_NAME  VARCHAR2(200) NOT NULL, 
    JOB_GROUP VARCHAR2(200) NOT NULL,
    JOB_LISTENER VARCHAR2(200) NOT NULL,
    PRIMARY KEY (JOB_NAME,JOB_GROUP,JOB_LISTENER),
    FOREIGN KEY (JOB_NAME,JOB_GROUP) 
	REFERENCES lj_JOB_DETAILS(JOB_NAME,JOB_GROUP)
);
CREATE TABLE lj_triggers
  (
    TRIGGER_NAME VARCHAR2(200) NOT NULL,
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    JOB_NAME  VARCHAR2(200) NOT NULL, 
    JOB_GROUP VARCHAR2(200) NOT NULL,
    IS_VOLATILE VARCHAR2(1) NOT NULL,
    DESCRIPTION VARCHAR2(250) NULL,
    NEXT_FIRE_TIME NUMBER(13) NULL,
    PREV_FIRE_TIME NUMBER(13) NULL,
    PRIORITY NUMBER(13) NULL,
    TRIGGER_STATE VARCHAR2(16) NOT NULL,
    TRIGGER_TYPE VARCHAR2(8) NOT NULL,
    START_TIME NUMBER(13) NOT NULL,
    END_TIME NUMBER(13) NULL,
    CALENDAR_NAME VARCHAR2(200) NULL,
    MISFIRE_INSTR NUMBER(2) NULL,
    JOB_DATA BLOB NULL,
    PRIMARY KEY (TRIGGER_NAME,TRIGGER_GROUP),
    FOREIGN KEY (JOB_NAME,JOB_GROUP) 
	REFERENCES lj_JOB_DETAILS(JOB_NAME,JOB_GROUP) 
);
CREATE TABLE lj_simple_triggers
  (
    TRIGGER_NAME VARCHAR2(200) NOT NULL,
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    REPEAT_COUNT NUMBER(7) NOT NULL,
    REPEAT_INTERVAL NUMBER(12) NOT NULL,
    TIMES_TRIGGERED NUMBER(10) NOT NULL,
    PRIMARY KEY (TRIGGER_NAME,TRIGGER_GROUP),
    FOREIGN KEY (TRIGGER_NAME,TRIGGER_GROUP) 
	REFERENCES lj_TRIGGERS(TRIGGER_NAME,TRIGGER_GROUP)
);
CREATE TABLE lj_cron_triggers
  (
    TRIGGER_NAME VARCHAR2(200) NOT NULL,
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    CRON_EXPRESSION VARCHAR2(120) NOT NULL,
    TIME_ZONE_ID VARCHAR2(80),
    PRIMARY KEY (TRIGGER_NAME,TRIGGER_GROUP),
    FOREIGN KEY (TRIGGER_NAME,TRIGGER_GROUP) 
	REFERENCES lj_TRIGGERS(TRIGGER_NAME,TRIGGER_GROUP)
);
CREATE TABLE lj_blob_triggers
  (
    TRIGGER_NAME VARCHAR2(200) NOT NULL,
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    BLOB_DATA BLOB NULL,
    PRIMARY KEY (TRIGGER_NAME,TRIGGER_GROUP),
    FOREIGN KEY (TRIGGER_NAME,TRIGGER_GROUP) 
        REFERENCES lj_TRIGGERS(TRIGGER_NAME,TRIGGER_GROUP)
);
CREATE TABLE lj_trigger_listeners
  (
    TRIGGER_NAME  VARCHAR2(200) NOT NULL, 
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    TRIGGER_LISTENER VARCHAR2(200) NOT NULL,
    PRIMARY KEY (TRIGGER_NAME,TRIGGER_GROUP,TRIGGER_LISTENER),
    FOREIGN KEY (TRIGGER_NAME,TRIGGER_GROUP) 
	REFERENCES lj_TRIGGERS(TRIGGER_NAME,TRIGGER_GROUP)
);
CREATE TABLE lj_calendars
  (
    CALENDAR_NAME  VARCHAR2(200) NOT NULL, 
    CALENDAR BLOB NOT NULL,
    PRIMARY KEY (CALENDAR_NAME)
);
CREATE TABLE lj_paused_trigger_grps
  (
    TRIGGER_GROUP  VARCHAR2(200) NOT NULL, 
    PRIMARY KEY (TRIGGER_GROUP)
);
CREATE TABLE lj_fired_triggers 
  (
    ENTRY_ID VARCHAR2(95) NOT NULL,
    TRIGGER_NAME VARCHAR2(200) NOT NULL,
    TRIGGER_GROUP VARCHAR2(200) NOT NULL,
    IS_VOLATILE VARCHAR2(1) NOT NULL,
    INSTANCE_NAME VARCHAR2(200) NOT NULL,
    FIRED_TIME NUMBER(13) NOT NULL,
    PRIORITY NUMBER(13) NOT NULL,
    STATE VARCHAR2(16) NOT NULL,
    JOB_NAME VARCHAR2(200) NULL,
    JOB_GROUP VARCHAR2(200) NULL,
    IS_STATEFUL VARCHAR2(1) NULL,
    REQUESTS_RECOVERY VARCHAR2(1) NULL,
    PRIMARY KEY (ENTRY_ID)
);
CREATE TABLE lj_scheduler_state 
  (
    INSTANCE_NAME VARCHAR2(200) NOT NULL,
    LAST_CHECKIN_TIME NUMBER(13) NOT NULL,
    CHECKIN_INTERVAL NUMBER(13) NOT NULL,
    PRIMARY KEY (INSTANCE_NAME)
);
CREATE TABLE lj_locks
  (
    LOCK_NAME  VARCHAR2(40) NOT NULL, 
    PRIMARY KEY (LOCK_NAME)
);
INSERT INTO lj_locks values('TRIGGER_ACCESS');
INSERT INTO lj_locks values('JOB_ACCESS');
INSERT INTO lj_locks values('CALENDAR_ACCESS');
INSERT INTO lj_locks values('STATE_ACCESS');
INSERT INTO lj_locks values('MISFIRE_ACCESS');
create index idx_lj_j_req_recovery on lj_job_details(REQUESTS_RECOVERY);
create index idx_lj_t_next_fire_time on lj_triggers(NEXT_FIRE_TIME);
create index idx_lj_t_state on lj_triggers(TRIGGER_STATE);
create index idx_lj_t_nft_st on lj_triggers(NEXT_FIRE_TIME,TRIGGER_STATE);
create index idx_lj_t_volatile on lj_triggers(IS_VOLATILE);
create index idx_lj_ft_trig_name on lj_fired_triggers(TRIGGER_NAME);
create index idx_lj_ft_trig_group on lj_fired_triggers(TRIGGER_GROUP);
create index idx_lj_ft_trig_nm_gp on lj_fired_triggers(TRIGGER_NAME,TRIGGER_GROUP);
create index idx_lj_ft_trig_volatile on lj_fired_triggers(IS_VOLATILE);
create index idx_lj_ft_trig_inst_name on lj_fired_triggers(INSTANCE_NAME);
create index idx_lj_ft_job_name on lj_fired_triggers(JOB_NAME);
create index idx_lj_ft_job_group on lj_fired_triggers(JOB_GROUP);
create index idx_lj_ft_job_stateful on lj_fired_triggers(IS_STATEFUL);
create index idx_lj_ft_job_req_recovery on lj_fired_triggers(REQUESTS_RECOVERY);



commit;
